<?php

namespace FiloBlu\Flow\Model\Actions\From;

use Exception;
use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\Product\Type;
use Magento\Catalog\Model\Product\Visibility;

/**
 * Class Addtoconfigurable
 * @package FiloBlu\Flow\Model\Actions\From
 */
class Addtoconfigurable
{
    /**
     * @var string
     */
    const ENTITY_TYPE_CODE = 'catalog_product';

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $productFactory;

    /**
     * @var
     */
    protected $passedVisibility;

    /**
     * @var \FiloBlu\Flow\Helper\Product
     */
    protected $helperProduct;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Eav\Model\ResourceModel\Entity\AttributeFactory
     */
    protected $attributeFactory;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\ProductFactory
     */
    protected $productResourceFactory;
    /**
     * @var \Magento\ConfigurableProduct\Model\Product\Type\ConfigurableFactory 
     */
    protected $typeConfigurableFactory;

    /**
     * Addtoconfigurable constructor.
     * @param \FiloBlu\Flow\Helper\Product $helperProduct
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Eav\Model\ResourceModel\Entity\AttributeFactory $attributeFactory
     * @param \Magento\Catalog\Model\ResourceModel\ProductFactory $productResourceFactory
     * @param \Magento\ConfigurableProduct\Model\Product\Type\ConfigurableFactory $typeConfigurableFactory
     */
    public function __construct(
        \FiloBlu\Flow\Helper\Product $helperProduct,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Eav\Model\ResourceModel\Entity\AttributeFactory $attributeFactory,
        \Magento\Catalog\Model\ResourceModel\ProductFactory $productResourceFactory,
        \Magento\ConfigurableProduct\Model\Product\Type\ConfigurableFactory $typeConfigurableFactory
    )
    {
        $this->helperProduct = $helperProduct;
        $this->productFactory = $productFactory;
        $this->storeManager = $storeManager;
        $this->attributeFactory = $attributeFactory;
        $this->productResourceFactory = $productResourceFactory;
        $this->typeConfigurableFactory = $typeConfigurableFactory;
    }

    /**
     * @param $simpleProduct
     * @param $value
     * @param $storeId
     * @param null $visibility
     * @throws Exception
     */
    public function process($simpleProduct, $value, $storeId, $visibility = null)
    {
        $this->passedVisibility = $visibility;

        if (empty($value) ) {
            throw new Exception("Invalid value supplied '{$value}'");
        }

        if(!$simpleProduct->getId()) {
            throw new Exception('Simple product is invalid');
        }

        if ($storeId !== 0) {
            throw new Exception('The function is avalaible only for the ADMIN store.');
        }

        $this->storeManager->setCurrentStore($storeId);

//      if (!preg_match("/^[0-9a-z_\-\.]+#([0-9a-z_]+([,][0-9a-z_]+)*)+$/i", $value)) {
//          return "Configurable SKU is not valid, please use only 0-9 a-z - . and _";
//      }

        $splitted = explode('#', $value);
        $configurableSku = trim($splitted[0]);
        $superAttributeIds = explode(',', $splitted[1]);
        if (count($splitted) > 2) {
            $parentField = trim($splitted[2]);
        }
        unset($splitted);

        if ($configurableSku === $simpleProduct->getSku()) {
            throw new Exception('Configurable SKU cannot be equal to Simple SKU');
        }

        //check if the simple product has a value for all the super attributes...
        $eavAttributeModel = $this->attributeFactory->create();
        $resourceProductModel = $this->productResourceFactory->create();
        $superValuesMap = array();

        foreach ($superAttributeIds as $attributeCode) {
            $attributeId = is_numeric($attributeCode) ? $attributeCode : $eavAttributeModel->getIdByCode('catalog_product', $attributeCode);
            $value = $resourceProductModel->getAttributeRawValue($simpleProduct->getId(), $attributeId, $storeId);
            // Values with 0 integer do not need to be blocked
            if (trim($value) === '') {
                $message = sprintf("Simple product #%s has no value for super attribute '%s'. All super attributes must have a value in the simple product.", $simpleProduct->getId(), $attributeId);
                throw new Exception($message);
            }

            $superValuesMap[$attributeId] = $value;
        }

        //getting the configurable product...
        $productId = $this->productFactory->create()->getIdBySku($configurableSku);
        $configurableProduct = $this->productFactory->create()->setStoreId($storeId)->load($productId);

        if (!$configurableProduct->getId()) {

            $configurableProduct = $this->_createConfigurableFromSimple($configurableSku, $superAttributeIds, $simpleProduct, $storeId);

            if (is_string($configurableProduct)) {
                return;
            }
        } else {
            $this->_syncAttributesFromSimpleToConfigurable($simpleProduct->getId(), $configurableProduct, $configurableProduct->getStoreIds());
        }

        // Getting current associated children...

        $childIds = $this->typeConfigurableFactory->create()->getChildrenIds($configurableProduct->getId());
        $childIds = $childIds[0];

        // Check if this simple product is already associated...

        if (in_array($simpleProduct->getId(), $childIds, false)) {
            //resetting visibility and status...
            $attributesData = [
                'status' => Status::STATUS_ENABLED,
                'visibility' => Visibility::VISIBILITY_NOT_VISIBLE
            ];
            // If i have the visibility forced, then I will force it
            if ($this->passedVisibility) {
                $attributesData['visibility'] = $this->passedVisibility;
            }

            if (!empty($parentField)) {
                $attributesData[$parentField] = $configurableSku;
            }

            \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Catalog\Model\Product\Action')->updateAttributes(
                [$simpleProduct->getId()], $attributesData, $storeId
            );

            return;
        }

        //check if already exists a product with the same super attributes values associated to the configurable product...
        $sameValuesProduct = $configurableProduct->getTypeInstance()->getProductByAttributes($superValuesMap, $configurableProduct);
        if (!is_null($sameValuesProduct)) {
            $message = sprintf('Product #%s with same super attributes values of simple #%s is already associated to the configurable product.', $sameValuesProduct->getId(), $simpleProduct->getId());
            throw new Exception($message);
        }

        //adding simple to configurable...
        $childIds[] = $simpleProduct->getId();
        \Magento\Framework\App\ObjectManager::getInstance()->create(\Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable::class)
            ->saveProducts($configurableProduct, $childIds);

        /* needed for correct index prices calculations: START */
        $shouldSaveConfigurable = false;
        if ($configurableProduct->getData('has_options') === 0) {
            $configurableProduct->setData('has_options', 1);
            $shouldSaveConfigurable = true;
        }

        if ($configurableProduct->getData('required_options') === 0) {
            $configurableProduct->setData('required_options', 1);
            $shouldSaveConfigurable = true;
        }

        if ($shouldSaveConfigurable) {
            $configurableProduct->setAssociatedProductIds($childIds);
            $configurableProduct->save();
        }
        /* needed for correct index prices calculations: END */

        //hide the simple product visibility...
        $attributesData = [
            'status' => Status::STATUS_ENABLED,
            'visibility' => Visibility::VISIBILITY_NOT_VISIBLE
        ];
        // If i have the visibility forced, then I will force it
        if ($this->passedVisibility) {
            $attributesData['visibility'] = $this->passedVisibility;
        }

        if (!empty($parentField)) {
            $attributesData[$parentField] = $configurableSku;
        }

        \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Catalog\Model\Product\Action::class)->updateAttributes(
            array($simpleProduct->getId()), $attributesData, $storeId
        );

        //setting configurable product price as the children's lower final price...
        /*
        $this->_prepareProductForSpecialPrice($configurableProduct->getId());
        if (\Magento\Framework\App\ObjectManager::getInstance()->create('\FiloBlu\Flow\Helper\Product')->getUpdateConfigurablePrice()) {

            $this->_setConfigurablePrice($configurableProduct->getId(), 0);
            foreach ($configurableProduct->getStoreIds() as $row) { // $storeId is already used

                $this->_setConfigurablePrice($configurableProduct->getId(), $row, Mage::getModel('core/store')->load($row)->getWebsiteId());
            }
        }
         *
         */

        return;
    }

    /**
     * @param $configurableSku
     * @param $superAttributeIds
     * @param $simpleProduct
     * @param int $storeId
     * @return mixed
     */
    protected function _createConfigurableFromSimple($configurableSku, $superAttributeIds, $simpleProduct, $storeId = 0)
    {
        $configurableProduct = $this->productFactory->create();
        $configurableProduct->setTypeId(\Magento\ConfigurableProduct\Model\Product\Type\Configurable::TYPE_CODE);
        $configurableProduct->setStoreId($storeId);

        //setting the super attributes to the configurable product...
        $superAttributes =[];
        $eavAttributeModel = \Magento\Framework\App\ObjectManager::getInstance()->create(\Magento\Eav\Model\ResourceModel\Entity\Attribute::class);
        foreach ($superAttributeIds as $superAttributeId) {
            $attributeId = is_numeric($superAttributeId) ? $superAttributeId : $eavAttributeModel->getIdByCode('catalog_product', $superAttributeId);
            $attribute = \Magento\Framework\App\ObjectManager::getInstance()->create(\Magento\Eav\Model\Entity\Attribute::class);
            $attribute->load($attributeId);
            if (!$attribute->getId()) {
                return sprintf('Super attribute with id #%d not found.', $superAttributeId);
            }

            $configurableAttribute = \Magento\Framework\App\ObjectManager::getInstance()->create(\Magento\ConfigurableProduct\Model\Product\Type\Configurable\Attribute::class)->setProductAttribute($attribute);
//            $configurableAttribute = Mage::getModel('catalog/product_type_configurable_attribute')->setProductAttribute($attribute);
            $superAttributes[] = [
                'id' => $configurableAttribute->getId(),
                'label' => $configurableAttribute->getLabel(),
                'position' => $attribute->getPosition(),
                'values' => $configurableAttribute->getPrices() ?: [],
                'attribute_id' => $attribute->getId(),
                'attribute_code' => $attribute->getAttributeCode(),
                'frontend_label' => $attribute->getFrontend()->getLabel(),
                'use_default' => 1
            ];
        }

        $configurableProduct->setCanSaveConfigurableAttributes(true);
        $configurableProduct->setConfigurableAttributesData($superAttributes);

        //copy all the attributes of specified groups from the simple product...
        $attributesToCopy = \Magento\Framework\App\ObjectManager::getInstance()->create(\FiloBlu\Flow\Model\System\ProductAttributes::class)->getAttributesToCopy()->getColumnValues('attribute_code');
        $excludedAttributes = array_diff($attributesToCopy, $this->helperProduct->getSimpleToConfigurableSyncAttributes());
        //$this->helper_product->getSimpleToConfigurableProductAttributesToExclude();
        $attributesToCopy = array_diff($attributesToCopy, $excludedAttributes);
        foreach ($attributesToCopy as $attributeCode) {
            $configurableProduct->setData($attributeCode, $simpleProduct->getData($attributeCode));
            if ($simpleProduct->hasData($attributeCode)) {
                $configurableProduct->setData($attributeCode, $simpleProduct->getData($attributeCode));
            }
        }

        //setting new attributes...
        if (!$configurableProduct->getName()) {
            $configurableProduct->setName($simpleProduct->getName());
        }

        if (!$configurableProduct->getDescription()) {
            $configurableProduct->setDescription($simpleProduct->getDescription());
        }

        if (!$configurableProduct->getShortDescription()) {
            $configurableProduct->setShortDescription($simpleProduct->getShortDescription());
        }

        if (!$configurableProduct->getWeight()) {
            $configurableProduct->setWeight(0);
        }


        // If i have the visibility forced, then I will force it
        // else fallback to VISIBILITY_BOTH
        if ($this->passedVisibility) {
            $visibility = $this->passedVisibility;
        } else {
            $visibility = Visibility::VISIBILITY_BOTH;
        }

        $configurableProduct->setSku($configurableSku);
        $configurableProduct->setStatus(Status::STATUS_DISABLED);
        $configurableProduct->setVisibility(Visibility::VISIBILITY_BOTH);
        $configurableProduct->setAttributeSetId($simpleProduct->getAttributeSetId());
        $configurableProduct->setTaxClassId($simpleProduct->getTaxClassId());
        $configurableProduct->setPrice(0);
        $configurableProduct->setPriceView(0);
        $configurableProduct->setCategoryIds($simpleProduct->getCategoryIds());
        $configurableProduct->setWebsiteIds($simpleProduct->getWebsiteIds());
        $configurableProduct->setStockData(['is_in_stock' => 1]);
        /* needed for correct index prices calculations: START */
        $configurableProduct->setData('has_options', 1);
        $configurableProduct->setData('required_options', 1);
        $configurableProduct->setUseDefault(1);

        /* needed for correct index prices calculations: END */
        try {
            $configurableProduct->save();
        } catch (Exception $e) {
            if ($e->getMessage() === 'URL key for specified store already exists.') {
                $urlKey = $configurableProduct->getUrlKey();
                $newUrlKey = $urlKey . '-' . $configurableProduct->getId();
                $configurableProduct->setUrlKey($newUrlKey);
                $configurableProduct->save();
            }

        }

        //copy localized attributes from the other stores...
        $this->_syncAttributesFromSimpleToConfigurable($simpleProduct->getId(), $configurableProduct, $configurableProduct->getStoreIds());
        return $configurableProduct;
    }

    /**
     * @param $simpleId
     * @param $configurableProduct
     * @param $storeIds
     * @return bool
     */
    protected function _syncAttributesFromSimpleToConfigurable($simpleId, $configurableProduct, $storeIds)
    {
        $attributesToCopy = $this->helperProduct->getSimpleToConfigurableSyncAttributes();

        if (empty($attributesToCopy)) {
            return false;
        }

        $superAttributesCode = array();
        // se il configurabile perde i prodotti semplici associati, torna ad essere un semplice, e a riga 275 darà errore - va gestito il caso
        if ($configurableProduct->getTypeId() !== Type::DEFAULT_TYPE) {
            foreach ($configurableProduct->getTypeInstance()->getConfigurableAttributesAsArray($configurableProduct) as $superAttribute) {
                $superAttributesCode[] = $superAttribute['attribute_code'];
            }
        }

        $attributesToCopy = array_diff($attributesToCopy, $superAttributesCode);
        if (empty($attributesToCopy)) {
            return false;
        }

        $configurableId = $configurableProduct->getId();

        array_unshift($storeIds, '0');

        foreach ($storeIds as $storeId) {

            $simple = $this->productFactory->create()->setStoreId($storeId)->load($simpleId);
            $attributesData = [];
            foreach ($attributesToCopy as $attributeCode) {
                // need to copy only the values actually present for the stores, excluding "use default"
                if ('url_key' === $attributeCode && $this->helperProduct->getUpdateConfigurableUrl()) {
                    $name = $simple->getData('name');
                    $confUrlkey = $configurableProduct->formatUrlKey($name . '-' . $storeId . $configurableProduct->getId());
                    $attributesData[$attributeCode] = $confUrlkey;
                } else {
                    $attributesData[$attributeCode] = $simple->getData($attributeCode);
                }
            }

            \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Catalog\Model\Product\Action::class)->updateAttributes([$configurableId], $attributesData, $storeId);
            unset($simple, $attributesData);
        }

        return true;
    }

    /**
     * @param $productId
     */
    protected function _prepareProductForSpecialPrice($productId)
    {
        if (!\Magento\Framework\App\ObjectManager::getInstance()->create(\Magento\Catalog\Model\ResourceModel\Product::class)->getAttributeRawValue($productId, 'special_price', .0)) {
            \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Catalog\Model\Product\Action::class)->updateAttributes(
                [$productId], ['special_price' => null], 0
            );
        }
    }

    /**
     * @param $configurableId
     * @param $storeId
     * @param null $websiteId
     */
    protected function _setConfigurablePrice($configurableId, $storeId, $websiteId = null)
    {

    }

}

