<?php
/**
 * Copyright © 2015 FiloBlu. All rights reserved.
 */

namespace FiloBlu\Flow\Model;

/**
 * Inboundflowtab inboundflow model
 */
class Channel extends \Magento\Framework\Model\AbstractModel
{

    /**
     * @var \Monolog\Logger
     */
    protected $_logger;

    /**
     * Channel constructor.
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    )
    {
        $this->_logger = $loggerProvider->getLogger();
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\Channel');
    }

    /**
     * @param int $id
     * @param null $field
     * @return $this|\Magento\Framework\Model\AbstractModel
     */
    public function load($id, $field = null)
    {
        parent::load($id, $field);

        $json = $this->getData('data');

        try {
            $object = json_decode($json, false);
        } catch (\Exception $e) {
            $this->_logger->info($e->getMessage());
            $this->_messageManager->addError($e->getMessage());
            return $this;
        }

        if (isset($object->config)) {
            $this->setData('config', (Array)$object->config);
        }

        if (isset($object->connector)) {
            $this->setData('connector', (Array)$object->connector);
        }

        if (isset($object->parser)) {
            $this->setData('parser', (Array)$object->parser);
        }

        if (isset($object->map)) {
            $this->setData('map', json_encode($object->map));
        }

        if (isset($object->use_map)) {
            $this->setData('use_map', $object->use_map ? 1 : 0);
        }

        return $this;
    }

    /**
     * @return \Magento\Framework\Model\AbstractModel
     * @throws \Exception
     */
    public function save()
    {
        $data = $this->getData();

        $obj = new \stdClass();

        if ($data['config']) {
            $obj->config = (object)$data['config'];
        }

        if ($data['connector']) {
            $obj->connector = (object)$data['connector'];
        }

        if ($data['parser']) {
            $obj->parser = (object)$data['parser'];
        }

        if ($data['map']) {
            $obj->map = json_decode($data['map'], JSON_FORCE_OBJECT);
        }

        if ($data['use_map']) {
            $obj->use_map = $data['use_map'];
        }

        $error = json_last_error();

        if ($error) {
            throw new \Exception("no valid json map");
        }

        $obj_decoded = str_replace('null', '', json_encode($obj));

        $this->setData('data', $obj_decoded);

        return parent::save();

    }

    /**
     * @return bool
     */
    public function usesMap()
    {
        return $this->getData('use_map') ? true : false;
    }

}
